@@ -109,6 +109,45 @@ typedef struct
 }
 DWARF2_Internal_ARange;
 
+enum dwarf_section_display_enum
+{
+  abbrev = 0,
+  aranges,
+  frame,
+  info,
+  line,
+  pubnames,
+  eh_frame,
+  macinfo,
+  macro,
+  str,
+  loc,
+  pubtypes,
+  ranges,
+  static_func,
+  static_vars,
+  types,
+  weaknames,
+  gdb_index,
+  trace_info,
+  trace_abbrev,
+  trace_aranges,
+  info_dwo,
+  abbrev_dwo,
+  types_dwo,
+  line_dwo,
+  loc_dwo,
+  macro_dwo,
+  macinfo_dwo,
+  str_dwo,
+  str_index,
+  str_index_dwo,
+  debug_addr,
+  dwp_cu_index,
+  dwp_tu_index,
+  max
+};
+
 struct dwarf_section
 {
   /* A debug section has a different name when it's stored compressed
@@ -121,6 +160,7 @@ struct dwarf_section
   unsigned char *start;
   dwarf_vma address;
   dwarf_size_type size;
+  enum dwarf_section_display_enum abbrev_sec;
 };
 
 /* A structure containing the name of a debug section
@@ -133,30 +173,6 @@ struct dwarf_section_display
   unsigned int relocate : 1;
 };
 
-enum dwarf_section_display_enum
-{
-  abbrev = 0,
-  aranges,
-  frame,
-  info,
-  line,
-  pubnames,
-  eh_frame,
-  macinfo,
-  str,
-  loc,
-  pubtypes,
-  ranges,
-  static_func,
-  static_vars,
-  types,
-  weaknames,
-  trace_info,
-  trace_abbrev,
-  trace_aranges,
-  max
-};
-
 extern struct dwarf_section_display debug_displays [];
 
 /* This structure records the information that
@@ -168,6 +184,12 @@ typedef struct
   int            dwarf_version;
   dwarf_vma	 cu_offset;
   dwarf_vma	 base_address;
+  /* This field is filled in when reading the attribute DW_AT_GNU_addr_base and
+     is used with the form DW_AT_GNU_FORM_addr_index.  */
+  dwarf_vma	 addr_base;
+  /* This field is filled in when reading the attribute DW_AT_GNU_ranges_base and
+     is used when calculating ranges.  */
+  dwarf_vma	 ranges_base;
   /* This is an array of offsets to the location list table.  */
   dwarf_vma *    loc_offsets;
   int *          have_frame_base;
@@ -198,8 +220,15 @@ extern int do_gdb_index;
 extern int do_trace_info;
 extern int do_trace_abbrevs;
 extern int do_trace_aranges;
+extern int do_debug_addr;
+extern int do_debug_cu_index;
 extern int do_wide;
 
+extern int dwarf_cutoff_level;
+extern unsigned long dwarf_start_die;
+
+extern int dwarf_check;
+
 extern void init_dwarf_regnames (unsigned int);
 extern void init_dwarf_regnames_i386 (void);
 extern void init_dwarf_regnames_x86_64 (void);
@@ -213,6 +242,8 @@ extern void dwarf_select_sections_by_names (const char *);
 extern void dwarf_select_sections_by_letters (const char *);
 extern void dwarf_select_sections_all (void);
 
+unsigned int * find_cu_tu_set (void *, unsigned int);
+
 void * cmalloc (size_t, size_t);
 void * xcmalloc (size_t, size_t);
 void * xcrealloc (void *, size_t, size_t);
