@@ -68,6 +68,8 @@ enum option_values
   OPTION_TDATA,
   OPTION_TTEXT,
   OPTION_TTEXT_SEGMENT,
+  OPTION_TRODATA_SEGMENT,
+  OPTION_TLDATA_SEGMENT,
   OPTION_TRADITIONAL_FORMAT,
   OPTION_UR,
   OPTION_VERBOSE,
